// ComecocosView.cpp : implementation of the CComecocosView class
//

#include "stdafx.h"
#include "Comecocos.h"

#include "ComecocosDoc.h"
#include "ComecocosView.h"
#include "DialogoRecord.h"
#include "MainFrm.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



//**********************************************************************************************//
//*             Seleccionamos los mensajes que vamos a escuchar en esta clase                  *//
//**********************************************************************************************//
IMPLEMENT_DYNCREATE(CComecocosView, CView)

BEGIN_MESSAGE_MAP(CComecocosView, CView)
	//{{AFX_MSG_MAP(CComecocosView)
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




//**********************************************************************************************//
//*                               Constructor y destructor                                     *//
//**********************************************************************************************//
CComecocosView::CComecocosView()
{
   //Inicializamos variables
   oldTimer = 0;
   oldGraficos3D = false;
   relojActivo = false;

   //Carga por defecto los graficos clasicos
   if (!(Btiles.LoadBitmap(BTiles) && Bfantasmas.LoadBitmap(BFantasmas) && Bpacman.LoadBitmap(BPacman))) //Carga BTiles de los recursos
      exit(-1);
 
   
}

CComecocosView::~CComecocosView()
{
}


//**********************************************************************************************//
//*                                                                                            *//
//**********************************************************************************************//
BOOL CComecocosView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
   
   return CView::PreCreateWindow(cs);
}


//**********************************************************************************************//
//*                            Cambia el tipo de graficos                                      *//
//**********************************************************************************************//
void CComecocosView::cambiaGraficos()
{
   
   oldGraficos3D = !oldGraficos3D;
   
   //Borramos los que habiamos cargado
   Btiles.DeleteObject();
   Bfantasmas.DeleteObject();
   Bpacman.DeleteObject();


   //Si no esta activada la opcion 3D activamos los graficos normales
   if (oldGraficos3D == false)
   {
      Btiles.LoadBitmap(BTiles);
      Bfantasmas.LoadBitmap(BFantasmas);
      Bpacman.LoadBitmap(BPacman);
   }
   else
   {
      Btiles.LoadBitmap(BTiles3D);
      Bfantasmas.LoadBitmap(BFantasmas3D);
      Bpacman.LoadBitmap(BPacman3D);
   }

   InvalidateRect(NULL, true);
}




//**********************************************************************************************//
//*                           Metodo de dibujado de toda la pantalla                           *//
//**********************************************************************************************//
void CComecocosView::OnDraw(CDC* pDC)
{ 
   //Activamos el reloj, para que cada 30 milisegundos se ejecute la funcion OnTimer
   if (!relojActivo)
   {
      setTimer(40);

      //Inicializamos el menu
      CMainFrame *frame = (CMainFrame *) GetParentFrame();
      frame->activaMenu();
   }

   pantallaBorrada = false;
   
   //Pintamos la pantalla de negro
   CRect miRect;
   CBrush Pincel;

   Pincel.CreateSolidBrush (RGB(0,0,0));//Creamos pincel slido
   GetClientRect(miRect);//Recogemos el rea cliente.
   pDC->FillRect(miRect,&Pincel);//Rellenamos el rectngulo


   //Creamos un puntero a la clase Documento
   CComecocosDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
  

   //Si el juego esta en marcha, pausado, o en game over, mostramos la pantalla y los personajes
   if (pDoc->getEstadoJuego() > 0 && pDoc->getEstadoJuego() < 3)
   {
      dibujaMapaPantalla();
   }

   //Si esta pidiendo el nombre del record
   else if (pDoc->getEstadoJuego() == 5)
   {
      dibujaMapaPantalla();
      dibujaPersonajes();      
   }
}


//**********************************************************************************************//
//*          Dibuja en pantalla el mapeado del juego que esta en la matriz mapaPantalla        *//
//**********************************************************************************************//
void CComecocosView::dibujaMapaPantalla()
{
   int i,j;
   char dato;
   CBitmap tiles; // Crea un objeto CBitmap
   CDC dcMemory ; // Contexto de dispositivo
   int nX, nY;
   CDC *pDC;
   CComecocosDoc* pDoc;
   CBitmap* pOldBitmap;

   //Creamos un puntero a la clase Documento
   pDoc = GetDocument();
	ASSERT_VALID(pDoc);

   pDC = GetDC(); //Obtenemos el contexto de dispositivo

   dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

   // Selecciona el bitmap en el contexto de dispositivo en memoria
   pOldBitmap = dcMemory.SelectObject(&Btiles);

   //Recorremos toda la matriz de pantalla y la vamos dibujando casilla a casilla
   for (j=0; j<16; j++)
   {
      for (i=0; i<16; i++)
      {           
         //Recogemos que tipo de casilla es
         dato = pDoc->mapaPantalla[i][j];
            
         //Calculamos en que posicion del Bitmap se encuentra la casilla 'dato'
         nX = dato % 2;
         nX *= 32;

         nY = dato / 2;
         nY *= 32;

         //Dibujamos la casilla en la pantalla
         pDC->BitBlt(i*32, j*32, 32, 32, &dcMemory, nX, nY, SRCCOPY);
      }
   }
      
   dcMemory.SelectObject(pOldBitmap); //Dejamos el bitmap que teniamos antes seleccionado

}



//**********************************************************************************************//
//*          Dibuja en pantalla el mapeado del juego que esta en la matriz mapaPantalla        *//
//**********************************************************************************************//
void CComecocosView::dibujaPuntuaciones()
{
   int i;
   int tiempoActual;
   CDC *pDC;
   CComecocosDoc* pDoc;
   CRect rec;
   char puntos[10];
   CBitmap* pOldBitmap;
   CDC dcMemory ; // Contexto de dispositivo


   //Creamos un puntero a la clase Documento
   pDoc = GetDocument();
	ASSERT_VALID(pDoc);

   pDC = GetDC(); //Obtenemos el contexto de dispositivo

   dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

   // Selecciona el bitmap en el contexto de dispositivo en memoria
   pOldBitmap = dcMemory.SelectObject(&Bpacman);


   //Fondo negro fuente blanca
   COLORREF oldbk = pDC->GetBkColor();
   COLORREF oldtext = pDC->GetTextColor();

   pDC->SetBkColor(RGB(0,0,0));
   pDC->SetTextColor(RGB(255,255,255));

   tiempoActual = pDoc->pacman.contador;

   //Escribimos "PUNTUACIONES:"
   rec.top = 100;

   i = 0;
   while (i<5 && pDoc->vectorPuntuaciones[i].puntos != -1)
   {
      rec.top += 40;
      rec.bottom = rec.top + 25;

      rec.left = 250;
      rec.right = rec.left + 150;
      pDC->DrawText(pDoc->vectorPuntuaciones[i].nombre, rec, DT_LEFT);


      if (i==1) dcMemory.SelectObject(&Bfantasmas);
      if (i>0) pDC->BitBlt(rec.left - 50, rec.top - 5, 32, 32, &dcMemory, (((tiempoActual/50)%2)== 0)*64 + 32, (i-1)*32, SRCCOPY);
      else pDC->BitBlt(rec.left - 50, rec.top - 5, 32, 32, &dcMemory, (2-((tiempoActual/5)%3))*32, 32, SRCCOPY);


      itoa(pDoc->vectorPuntuaciones[i].puntos,puntos, 10);
      rec.left = 350;
      rec.right = rec.left + 50;
      pDC->DrawText(puntos, rec, DT_RIGHT);
    
      i++;
   }


   dcMemory.SelectObject(&Bpacman);

   if (tiempoActual < 1000 && tiempoActual > 500)
   {
      pDC->BitBlt(((tiempoActual-800)*3), 400, 32, 32, &dcMemory, (2-((tiempoActual/5)%3))*32, 3*32, SRCCOPY);

      dcMemory.SelectObject(&Btiles);
      pDC->BitBlt(((tiempoActual-800)*3)+32, 400, 5, 32, &dcMemory, 0, 0, SRCCOPY);

      dcMemory.SelectObject(&Bfantasmas);
      for (i=0; i<4; i++)
         pDC->BitBlt(((tiempoActual-800)*3)+(i+3)*50, 400, 32, 32, &dcMemory, 3*32, i*32, SRCCOPY);
   }

   else if (tiempoActual < 500 && tiempoActual > 0)
   {
      pDC->BitBlt(((400-tiempoActual)*3)-7*32, 400, 32, 32, &dcMemory, (2-((tiempoActual/5)%3))*32, 1*32, SRCCOPY);

      dcMemory.SelectObject(&Btiles);
      pDC->BitBlt(((400-tiempoActual)*3)-5-7*32, 400, 5, 32, &dcMemory, 0, 0, SRCCOPY);

      dcMemory.SelectObject(&Bfantasmas);
      for (i=0; i<4; i++)
         pDC->BitBlt(((400-tiempoActual)*3)-(i)*50, 400, 32, 32, &dcMemory, 4*32, i*32, SRCCOPY);
   }



   dcMemory.SelectObject(pOldBitmap);
}


//**********************************************************************************************//
//*                        Activa el reloj a la frecuencia que le digamos                      *//
//**********************************************************************************************//
void CComecocosView::setTimer(int ms)
{
   //Si existia un reloj, lo matamos para volver a crearlo
   if (relojActivo) 
   {
      KillTimer(ID_TIMER);
   }

   SetTimer(ID_TIMER, ms, 0);
   relojActivo = true;
}


//**********************************************************************************************//
//*          Dibuja en pantalla el mapeado del juego que esta en la matriz mapaPantalla        *//
//**********************************************************************************************//
void CComecocosView::cambiaPuertas()
{
   int i,j;
   char dato;
   CBitmap tiles; // Crea un objeto CBitmap
   CDC dcMemory ; // Contexto de dispositivo
   int nX, nY;
   CDC *pDC;
   CComecocosDoc* pDoc;
   CBitmap* pOldBitmap;

   //Creamos un puntero a la clase Documento
   pDoc = GetDocument();
	ASSERT_VALID(pDoc);

   pDC = GetDC(); //Obtenemos el contexto de dispositivo

   dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

   // Selecciona el bitmap en el contexto de dispositivo en memoria
   pOldBitmap = dcMemory.SelectObject(&Btiles);

   //Recorremos toda la matriz de pantalla y la vamos dibujando casilla a casilla
   for (j=0; j<16; j++)
   {
      for (i=0; i<16; i++)
      {
         if (pDoc->mapaPantalla[i][j] > 25 && pDoc->mapaPantalla[i][j] < 30)
         {
            switch (pDoc->mapaPantalla[i][j]-26)
            {
               case 0: pDoc->mapaPantalla[i][j] = 28; break;
               case 1: pDoc->mapaPantalla[i][j] = 29; break;
               case 2: pDoc->mapaPantalla[i][j] = 26; break;
               case 3: pDoc->mapaPantalla[i][j] = 27; break;
            }
            
            //Recogemos que tipo de casilla es
            dato = pDoc->mapaPantalla[i][j];
           
            //Calculamos en que posicion del Bitmap se encuentra la casilla 'dato'
            nX = dato % 2;
            nX *= 32;
   
            nY = dato / 2;
            nY *= 32;
   
            //Dibujamos la casilla en la pantalla
            pDC->BitBlt(i*32, j*32, 32, 32, &dcMemory, nX, nY, SRCCOPY);
         }
      }
   }
      
   dcMemory.SelectObject(pOldBitmap); //Dejamos el bitmap que teniamos antes seleccionado

}


//**********************************************************************************************//
//*                               Dibuja los personajes en la pantalla                         *//
//**********************************************************************************************//
void CComecocosView::dibujaPersonajes()
{
   char dato;
   CDC* pDC;
   CRect rec;
   CBrush lapiz;
   CBitmap* pOldBitmap;
   CDC dcMemory ; // Contexto de dispositivo
   int nX, nY, i, j;
   int ii, jj, f;
   char puntos[25];
   char vidas[25];


   lapiz.CreateSolidBrush(RGB(0,0,0));
   
   pDC = GetDC();
   
	CComecocosDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

   dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

   // Selecciona el bitmap en el contexto de dispositivo en memoria
   pOldBitmap = dcMemory.SelectObject(&Btiles);


   //******************************************************//
   //Borramos las casillas que ocupaban antes los fantasmas
   //******************************************************//
   for (f=0; f<4; f++)
   {
      //Dibujamos la casilla actual
      ii = pDoc->enemigo[f].x / 32;
      jj = pDoc->enemigo[f].y / 32;

      for (j = jj-1; j < jj+2; j++)
      {
         for (i = ii-1; i < ii+2; i++)
         {
            if (i > -1 && i < 16 && j > -1 && j < 16)
            {
               dato = pDoc->mapaPantalla[i][j];
 
               nX = dato % 2;
               nX *= 32;
   
               nY = dato / 2;
               nY *= 32;
      
               pDC->BitBlt(i*32, j*32, 32, 32, &dcMemory, nX, nY, SRCCOPY);
            }
         }
      }
   }

   //*************************************************//
   //Borramos las casillas que ocupaba antes el pacman
   //*************************************************//
   ii = pDoc->pacman.x / 32;
   jj = pDoc->pacman.y / 32;

   for (j = jj-1; j < jj+2; j++)
   {
      for (i = ii-1; i < ii+2; i++)
      {
         if (i > -1 && i < 16 && j > -1 && j < 16)
         {
            dato = pDoc->mapaPantalla[i][j];
   
            nX = dato % 2;
            nX *= 32;
   
            nY = dato / 2;
            nY *= 32;
      
            pDC->BitBlt(i*32, j*32, 32, 32, &dcMemory, nX, nY, SRCCOPY);
         }

         //Si se sale por la derecha o por la izquierda tenemos que borrarlo
         else if ((i > 15 || i < 0) && j == jj)
         {
            rec.left = i*32;
            rec.top = j*32;

            rec.right = rec.left + 32;
            rec.bottom = rec.top + 32;

            pDC->BitBlt(rec.left, rec.top, 32, 32, &dcMemory, 0, 0, SRCCOPY); //SRCPAINT); //SRCCOPY);

            //Si estamos en el lado izquierdo ponemos el 'tile' de la derecha
            if (i<0)
            {
               //Borramos los restos que hayan quedado en el lado derecho
               rec.left = 16*32;
               rec.top = j*32;
  
               rec.right = rec.left + 32;
               rec.bottom = rec.top + 32;

               pDC->BitBlt(rec.left, rec.top, 32, 32, &dcMemory, 0, 0, SRCCOPY); //SRCPAINT); //SRCCOPY);

               //Ponemos el tile derecho
               dato = pDoc->mapaPantalla[15][j];
   
               if (dato > 25) dato = 0;
   
               nX = dato % 2;
               nX *= 32;
   
               nY = dato / 2;
               nY *= 32;
      
               pDC->BitBlt(15*32, j*32, 32, 32, &dcMemory, nX, nY, SRCCOPY);
            }

            //Si estamos en el lado derecho ponemos el 'tile' de la izquierda
            else if (i>15)
            {
               dato = pDoc->mapaPantalla[0][j];
   
               nX = dato % 2;
               nX *= 32;
  
               nY = dato / 2;
               nY *= 32;
      
               pDC->BitBlt(0, j*32, 32, 32, &dcMemory, nX, nY, SRCCOPY);
            }
         }
      }
   }



   //Si la fruta se va, pintamos lo que habia antes de poner la fruta
   if (pDoc->tiempoFruta == 0 && pDoc->tipoFruta != 0)
   {
      i = pDoc->posicionXFruta*32;
      j = pDoc->posicionYFruta*32;

      dato = pDoc->mapaPantalla[i][j];
   
      if (dato > 25) dato = 0;
   
      nX = dato % 2;
      nX *= 32;
   
      nY = dato / 2;
      nY *= 32;

      pDC->BitBlt(i, j, 32, 32, &dcMemory, nX, nY, SRCCOPY);
   }

   else if ((pDoc->tipoFruta == 1 || pDoc->tipoFruta == 2) && pDoc->tiempoFruta > 5)
   {
      i = pDoc->posicionXFruta*32;
      j = pDoc->posicionYFruta*32;

      dato = pDoc->tipoFruta+29;
     
      nX = dato % 2;
      nX *= 32;
   
      nY = dato / 2;
      nY *= 32;

      pDC->BitBlt(i, j, 32, 32, &dcMemory, nX, nY, SRCCOPY);
   }
   

   //*************************//
   //Dibujamos los 4 fantasmas
   //*************************//

//   dcMemory.DeleteTempMap(); //.DeleteDC();
   dcMemory.SelectObject(&Bfantasmas);

   for (f=0; f<4; f++)
   {
      //Dibujamos el fantasma normal
      if (pDoc->enemigo[f].estado == 0)
      {
         nY = 32*pDoc->enemigo[f].tipo;
         nX = pDoc->enemigo[f].dir*32;
      }

      //Si su estado es miedoso
      else if (pDoc->enemigo[f].estado == 1)
      {
         if (pDoc->enemigo[f].contador < 50 && (pDoc->enemigo[f].contador % 6) < 3)
         {
            nY = 32*pDoc->enemigo[f].tipo;
            nX = pDoc->enemigo[f].dir*32;
         }
         else
         {
            nY = 32*pDoc->enemigo[f].tipo;
            nX = 4*32;
         }
      }
      //Si su estado es muerto
      else if (pDoc->enemigo[f].estado == 2)
      {
         nX = 32*5;
         nY = pDoc->enemigo[f].dir*32;
      }

      if (pDoc->enemigo[f].estado == 2) pDC->BitBlt(pDoc->enemigo[f].x, pDoc->enemigo[f].y, 32, 32, &dcMemory, nX, nY, SRCPAINT); 
      else pDC->BitBlt(pDoc->enemigo[f].x, pDoc->enemigo[f].y, 32, 32, &dcMemory, nX, nY, SRCPAINT); 
   }


   //*******************//
   //Dibujamos el pacman
   //*******************//
   dcMemory.SelectObject(&Bpacman);

   //Si tiene estado normal
   if (pDoc->pacman.estado == 0)
   {
      nX = 32*(int) (pDoc->pacman.tipo/3);
      nY = pDoc->pacman.dir*32;
   }

   //Si tiene estado muerto
   else if (pDoc->pacman.estado == 2)
   {
      nX = 32*(int) (pDoc->pacman.tipo/3);
      nY = (pDoc->pacman.dir/4)*32;
   }

   pDC->BitBlt(pDoc->pacman.x, pDoc->pacman.y, 32, 32, &dcMemory, nX, nY, SRCPAINT);



   //**********************************//
   //* Ponemos los puntos y las vidas *//
   //**********************************//

   dcMemory.SelectObject(&Btiles);

   //Borramos todo con un cuadrado negro
   rec.left = 16*32;
   rec.top = 0;
   rec.right = rec.left + 100;
   rec.bottom = rec.top + 225;

   for (nY = 0; nY < 225; nY+=32)
   {
      for (nX = 16*32; nX < 16*32 + 127; nX+=32)
      {
         pDC->BitBlt(nX, nY+32, 32, 32, &dcMemory, 0, 0, SRCPAINT); //SRCCOPY);
      }
   }

   dcMemory.SelectObject(&Bpacman);

//   pDC->FillRect(rec, &lapiz);


   //Fondo negro fuente blanca
   COLORREF oldbk = pDC->GetBkColor();
   COLORREF oldtext = pDC->GetTextColor();

   pDC->SetBkColor(RGB(0,0,0));
   pDC->SetTextColor(RGB(255,255,255));


   //Dibujamos un grafico de pacman
   rec.top = 50;
   rec.bottom = rec.top + 25;
   pDC->BitBlt(rec.left + 15 , rec.top - 8, 32, 32, &dcMemory, 0, 32, SRCCOPY);

   //Escribimos las vidas que nos quedan
   rec.right = rec.left + 75;
   if (pDoc->getVidas() < 0) strcpy(puntos, "M");
   else itoa(pDoc->getVidas(), puntos, 10);
   strcpy(vidas, "x  ");
   strcat(vidas, puntos);
   pDC->DrawText(vidas, rec, DT_RIGHT);


   //Escribimos "PUNTUACIONES:"
   rec.right = rec.left + 100;
   rec.top = 100;
   rec.bottom = rec.top + 25;
   pDC->DrawText(" PUNTUACION:", rec, DT_LEFT);

   //Escribimos los puntos
   rec.top = 125;
   rec.bottom = rec.top + 25;
   
   itoa(pDoc->getPuntuacion(), puntos, 10);
   pDC->DrawText(puntos, rec, DT_RIGHT);


   //Comprobamos si existen puertas
   if (pDoc->getPuertas() && pDoc->getEstadoJuego() == 1)
   {
      //Escribimos "PUNTUACIONES:"
      rec.right = rec.left + 100;
      rec.top = 175;
      rec.bottom = rec.top + 25;
      pDC->DrawText("Estado Puertas:", rec, DT_LEFT);

      //Escribimos los puntos
      rec.top = 200;
      rec.bottom = rec.top + 25;
         
      if (pDoc->pacman.contador <= 20) strcpy(puntos, "CAMBIO");
      else itoa(pDoc->pacman.contador, puntos, 10);

      pDC->DrawText(puntos, rec, DT_RIGHT);
   }


   //Reestablecemos todo como estaba antes
   pDC->SetBkColor(oldbk);
   pDC->SetTextColor(oldtext);
   dcMemory.SelectObject(pOldBitmap);

   dcMemory.DeleteTempMap(); //DeleteDC();
   //pDC->DeleteDC();

}


//**********************************************************************************************//
//*                     Esta funcion se ejecuta cada vez que salta el timer                    *//
//**********************************************************************************************//
void CComecocosView::dibujaFrase(char f) 
{
   /* Valores posibles de f:
     0-> Juego Terminado
     1-> Nivel Terminado
     10+x -> Nivel X
   */

   CDC* pDC;
   CBitmap aux, aux2; // Crea un objeto CBitmap
   CDC dcMemory ; // Contexto de dispositivo
   CBitmap* pOldBitmap;
   BITMAP bmpInfo;
   pDC = GetDC();

	CComecocosDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
 

   switch (f)
   {
      //Frase "JUEGO TERMINADO"
      case 0:
      {
         //Si tenemos las librerias cargadas dibujamos la pantalla
         if (aux.LoadBitmap(BFinJuego))
         {           
            aux.GetBitmap(&bmpInfo); // Rellena la estructura BITMAP
            dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

            // Selecciona el bitmap en el contexto de dispositivo en memoria
            pOldBitmap = dcMemory.SelectObject(&aux);

            pDC->BitBlt(40 , 150, bmpInfo.bmWidth, bmpInfo.bmHeight, &dcMemory, 0, 0, SRCCOPY);
         }
         
         break;
      }

      //Frase "NIVEL TERMINADO"
      case 1:
      {
         //Si tenemos las librerias cargadas dibujamos la pantalla
         if (aux.LoadBitmap(BFinNivel))
         {
            aux.GetBitmap(&bmpInfo); // Rellena la estructura BITMAP
            dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

            // Selecciona el bitmap en el contexto de dispositivo en memoria
            pOldBitmap = dcMemory.SelectObject(&aux);

            pDC->BitBlt(60 , 150, bmpInfo.bmWidth, bmpInfo.bmHeight, &dcMemory, 0, 0, SRCCOPY);
         }
         
         break;
      }


      //Aqui comprobamos si es mayor que 10 y mostramos Nivel X
      default:
      {
         //Como mucho 99 pantallas
         if (f > 10 && f < 109)
         {
            //Si tenemos las librerias cargadas dibujamos la pantalla
            if (aux.LoadBitmap(BNivel) && aux2.LoadBitmap(BNumeros))
            {
               int x, y;
               int numero;

               aux.GetBitmap(&bmpInfo); // Rellena la estructura BITMAP
               dcMemory.CreateCompatibleDC(pDC); // Crea un contexto de dispositivo compatible

               //Recogemos la altura y la anchura
               x = bmpInfo.bmWidth;
               y = bmpInfo.bmHeight;

               // Selecciona el bitmap en el contexto de dispositivo en memoria
               pOldBitmap = dcMemory.SelectObject(&aux);
               pDC->BitBlt(175 , 150, x, y, &dcMemory, 0, 0, SRCCOPY);


               aux2.GetBitmap(&bmpInfo); // Rellena la estructura BITMAP
               dcMemory.SelectObject(&aux2);

               numero = f - 10;
               y = 13;

               if (numero > 9) 
               {
                  pDC->BitBlt(175+x + 15 , 150-y, 32, bmpInfo.bmHeight, &dcMemory, (numero/10)*32, 0, SRCCOPY);
                  x += 32;
                  numero = numero % 10;
               }

               pDC->BitBlt(175+x + 15 , 150-y, 32, bmpInfo.bmHeight, &dcMemory, numero*32, 0, SRCCOPY);
            }           
         }
         
         break;
      }
   }

   //Dejamos el Bitmap que teniamos antes seleccionado
   dcMemory.SelectObject(pOldBitmap);


   if (f == 0)
   {
      int i, j;
      CDialogoRecord midialogo;

      i=0;
      while (i < 5 && pDoc->vectorPuntuaciones[i].puntos > pDoc->getPuntuacion()) i++;

      if (i < 4)
      {
         for (j = 4; j > i; j--)
         {
            pDoc->vectorPuntuaciones[j] = pDoc->vectorPuntuaciones[j-1];
         }
      }

      if (i < 5)
      {
         midialogo.DoModal();

         pDoc->vectorPuntuaciones[i].puntos = pDoc->getPuntuacion();
         strcpy(pDoc->vectorPuntuaciones[i].nombre, midialogo.m_nombre);
      }

      pDoc->pacman.contador = 1000;
   }
}

//**********************************************************************************************//
//*                     Esta funcion se ejecuta cada vez que salta el timer                    *//
//**********************************************************************************************//
void CComecocosView::OnTimer(UINT nIDEvent) 
{  
 	CComecocosDoc* pDoc = GetDocument();
   ASSERT_VALID(pDoc);

   //Si los graficos han cambiado, cargamos los nuevos graficos
   if ((pDoc->GRAFICOS > 0) != oldGraficos3D) cambiaGraficos();


   if (nIDEvent == ID_TIMER)
   {
      //Si el tiempo ha cambiado actualizamos el reloj
      if (oldTimer != pDoc->TIMER) 
      {
         oldTimer = pDoc->TIMER;
         setTimer(oldTimer);
      }

      //En funcion del estado actual de juego elegimos lo que hacer
      switch (pDoc->getEstadoJuego())
      {
         //Si estamos en el menu principal, mostramos los records
         case 0:
            pDoc->pacman.contador--;
            if (pDoc->pacman.contador <= 0)
            {
               pDoc->pacman.contador = 1000;
            }

            dibujaPuntuaciones();
            break;

         //Si es juego normal manejamos personajes y los dibujamos
         case 1:
            pDoc->manejaPacman();

            //Si el contador llega a 0 cambiamos y dibujamos las puertas direccionales
            if (pDoc->pacman.contador == 0) cambiaPuertas();
            
            pDoc->manejaEnemigos();
            dibujaPersonajes();
            break;

         //Si el juego esta pausado, manejamos personajes y redibujamos la pantalla entera 
         case 2:
            pDoc->pacman.contador--;
            if (pDoc->pacman.contador <= 0)
            {
               pDoc->pacman.contador = 0;
               pDoc->setEstadoJuego(1);
            }
            
            //Si aun no ha sido borrada la pantalla o esta a punto de empezar el juego
            if (!pantallaBorrada || pDoc->pacman.contador < 2)
            {
               pantallaBorrada = true;

               //Inicializamos para la proxima vez que empiece el nivel
               if (pDoc->pacman.contador < 2) pantallaBorrada = false;
               dibujaMapaPantalla();

            }
            dibujaPersonajes();
            if (pDoc->pacman.contador > 1) dibujaFrase(10+pDoc->getPantallaActual());
            break;

         //Si se ha terminado el juego, mostramos la frase de "Juego Terminado"
         case 3:           
            //dibujaFrase(0);  
            pDoc->pacman.contador = 0;
            pDoc->setEstadoJuego(5);            
            break;


         //Si se ha terminado el nivel, incrementamos el nivel y cargamos la pantalla de nuevo
         case 4:
            dibujaFrase(1);

            pDoc->pacman.contador--;
            if (pDoc->pacman.contador <= 0)
            {
               pDoc->pacman.contador = 0;
               pDoc->iniciaPantalla(pDoc->getPantallaActual()+1);

               //Si al cargar pantalla el estado es 0, es pq no existe esa pantalla, luego el juego se ha terminado
               if (pDoc->getEstadoJuego() == 0) 
               {
                  pDoc->pacman.contador = 0;
                  pDoc->setEstadoJuego(5);
               }
               
               InvalidateRect(NULL, true);              
            }            
            break;

         //Si se ha terminado el juego pq no hay mas pantallas mostramos la frase y miramos si hay record
         case 5:
            if (pDoc->pacman.contador < 1000)
            {
               pDoc->pacman.contador = 1001;
               dibujaFrase(0);
            }

            else if (pDoc->pacman.contador == 1000)
            {
               pDoc->setEstadoJuego(0);              
               InvalidateRect(NULL, true);              
            }            
            break;

         default: break;
      }
   }
	
	CView::OnTimer(nIDEvent);
}


//**********************************************************************************************//
//*                        Cuando pulsamos una tecla se activa la funcion                      *//
//**********************************************************************************************//
void CComecocosView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	CView::OnKeyDown(nChar, nRepCnt, nFlags);

   CComecocosDoc* pDoc = GetDocument();
   ASSERT_VALID(pDoc);

	switch(nChar)
   {
      //se puls la tecla flecha arriba
	   case VK_UP: pDoc->setTecla(0,true); break;
      
      //se puls la tecla flecha derecha
	   case VK_RIGHT: pDoc->setTecla(1,true); break;

      //se puls la tecla flecha abajo
	   case VK_DOWN: pDoc->setTecla(2,true); break;

      //se puls la tecla flecha izquierda
	   case VK_LEFT: pDoc->setTecla(3,true); break;
   }

}


//**********************************************************************************************//
//*                        Cuando soltamos una tecla se activa la funcion                      *//
//**********************************************************************************************//
void CComecocosView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
   // TODO: Add your message handler code here and/or call default
	CView::OnKeyUp(nChar, nRepCnt, nFlags);

   CComecocosDoc* pDoc = GetDocument();
   ASSERT_VALID(pDoc);

   switch(nChar)
   {
      //se solt la tecla flecha arriba
	   case VK_UP: pDoc->setTecla(0,false); break;
      
      //se solt la tecla flecha derecha
	   case VK_RIGHT: pDoc->setTecla(1,false); break;

      //se solt la tecla flecha abajo
	   case VK_DOWN: pDoc->setTecla(2,false); break;

      //se solt la tecla flecha izquierda
	   case VK_LEFT: pDoc->setTecla(3,false); break;
   }

}






/////////////////////////////////////////////////////////////////////////////
// CComecocosView diagnostics

#ifdef _DEBUG
void CComecocosView::AssertValid() const
{
	CView::AssertValid();
}

void CComecocosView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CComecocosDoc* CComecocosView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CComecocosDoc)));
	return (CComecocosDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CComecocosView message handlers


//**********************************************************************************************//
//*                        Al cerrarse la vista, liberamos la memoria                          *//
//**********************************************************************************************//
void CComecocosView::OnDestroy() 
{
	CView::OnDestroy();
	
   KillTimer(ID_TIMER);

   
   Btiles.DeleteObject();
   Bpacman.DeleteObject();
   Bfantasmas.DeleteObject();

}
